function gen_direct2_thread_conv(th, tw, f)
% Generates a thread function for the 2d direct convolution 
% for a given template size.
%
% NOTE: This file is not used in current FLCC version
%
% INPUTS:
%     th    height of the template
%     tw    width of the template
%     f     filename to write to
%
% Georgios Papamakarios (gpapamak@auth.gr)
% Electrical and Computer Engineering AUTH
% April 2011


fprintf(f,'template <class flccPrecision>\n');
fprintf(f,'void *convolution2d_thread_%dx%d (void *arg)\n', th, tw);
fprintf(f,'{\n');
fprintf(f,'\tconst corrParm<flccPrecision> *c = (corrParm<flccPrecision> *)arg;\n');
fprintf(f,'\tfor (int n=0; n<c->offset; n++) {\n');
fprintf(f,'\t\tconst int indx = c->initialPosition + n;\n');
fprintf(f,'\t\tconst int u = indx%%c->sw;\n');
fprintf(f,'\t\tconst int v = indx/c->sw;\n');
fprintf(f,'\t\tregister flccPrecision conv = 0.0;\n');
fprintf(f,'\t\tint i_indx = v*c->bw + u;\n');

for i=0:th-1
    for j=0:tw-1
        fprintf(f,'\t\t\t\tconv += c->image[i_indx+%d] * c->templat[%d];\n', j, th*tw-1-j-i*tw);
    end
    fprintf(f,'\t\t\ti_indx += c->bw;\n');
end

fprintf(f,'\t\tc->lcc[indx] = conv; }\n');
fprintf(f,'\tpthread_exit (NULL);\n');
fprintf(f,'}\n');

fprintf(f,'#ifndef NO_SINGLE_PREC \n');
fprintf(f,'template void *convolution2d_thread_%dx%d<float>  (void *arg);\n', th, tw);
fprintf(f,'#endif \n');
fprintf(f,'#ifndef NO_DOUBLE_PREC \n');
fprintf(f,'template void *convolution2d_thread_%dx%d<double> (void *arg);\n', th, tw);
fprintf(f,'#endif \n');
fprintf(f,'\n');
