function gen_direct3_thread_conv(th, tw, td, f)
% Generates a thread function for the 3d direct convolution 
% for a given template size.
%
% NOTE: This file is not used in current FLCC version
%
% INPUTS:
%     th    height of the template
%     tw    width of the template
%     td    depth of the template
%     f     filename to write to
%
% Georgios Papamakarios (gpapamak@auth.gr)
% Electrical and Computer Engineering AUTH
% April 2011


fprintf(f,'template <class flccPrecision>\n');
fprintf(f,'void *convolution3d_thread_%dx%dx%d (void *arg)\n', th, tw, td);
fprintf(f,'{\n');
fprintf(f,'\tconst corrParm<flccPrecision> *c = (corrParm<flccPrecision> *)arg;\n');
fprintf(f,'\tfor (int n=0; n<c->offset; n++) {\n');
fprintf(f,'\t\tconst int indx = c->initialPosition + n;\n');
fprintf(f,'\t\tconst int u = (indx%%(c->sd*c->sw))%%c->sd;\n');
fprintf(f,'\t\tconst int v = (indx%%(c->sd*c->sw))/c->sd;\n');
fprintf(f,'\t\tconst int w = indx/(c->sd*c->sw);\n');
fprintf(f,'\t\tregister flccPrecision conv = 0.0;\n');
fprintf(f,'\t\tint i_indx = w*c->bd*c->bw + v*c->bd + u;\n');

for i=0:th-1
    for j=0:tw-1
        for k=0:td-1
            fprintf(f,'\t\t\t\t\tconv += c->image[i_indx+%d] * c->templat[%d];\n', k, (th*tw*td-1)-(k+j*td+i*td*tw));
        end
        fprintf(f,'\t\t\t\ti_indx += c->bd;\n');
    end
    fprintf(f,'\t\t\ti_indx += c->bd*(c->bw-%d);\n', td);
end

fprintf(f,'\t\tc->lcc[indx] = conv; }\n');
fprintf(f,'\tpthread_exit (NULL);\n');
fprintf(f,'}\n');

fprintf(f,'#ifndef NO_SINGLE_PREC \n');
fprintf(f,'template void *convolution3d_thread_%dx%dx%d<float>  (void *arg);\n', th, tw, td);
fprintf(f,'#endif \n');
fprintf(f,'#ifndef NO_DOUBLE_PREC \n');
fprintf(f,'template void *convolution3d_thread_%dx%dx%d<double> (void *arg);\n', th, tw, td);
fprintf(f,'#endif \n');
fprintf(f,'\n');
