/* 
 * FILE:
 * FLCC_fftw_defs.h 
 * version 1.1
 *
 * DESCRIPTION:
 * These are the global fftw definitions for the FLCC library source files.
 *
 * AUTHORS:
 * Georgios Papamakarios 
 * Georgios Rizos
 *
 * Aristotle University of Thessaloniki
 * Facutly of Engineering
 * Department of Electrical and Computer Engineering
 *
 * DATE:
 * November 2011
 *
 * CONTACT INFO:
 * e-mail: gpapamak@auth.gr
 *         georgios.rizos12@imperial.ac.uk
 */

 
#ifndef FLCC_FFTW_DEFS_H
#define FLCC_FFTW_DEFS_H
 
 
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// System includes
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <fftw3.h>


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Data types definitions
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class flccPrecision>
struct padSigParm {
	int            initialPosition;
	int            offset;
	flccPrecision  *signal;
	flccPrecision  *tempSignal;
	int            h, w, d;
	int            sw, sd;
};

template <class flccPrecision>
struct makeConvParm {
	int            initialPosition;
	int            offset;
	flccPrecision  *sceneTimesT;
	flccPrecision  *conv;
	int            sw, sd;
	int            SW, SD;
};

template <class flccPrecision>
struct  permParm {
	int            initialPosition;
	int            offset;
	flccPrecision  *templat; 
	flccPrecision  *tempTemplate; 
	flccPrecision  *Bt;
	flccPrecision  mean, std;
	int            tw, th, td;
	int            sw, sd;
};

template <class flccPrecision>
struct tranParm {
	int            initialPosition;
	int            offset;
	flccPrecision  *scene;
	flccPrecision  *sceneSq;
	flccPrecision  *image;
	int            iw, ih, id;
	int            sw, sd;
};

template <class fftwComplexPrecision>
struct prodParm {
	int                  initialPosition;
	int                  offset;
	fftwComplexPrecision *prod_fft;
	fftwComplexPrecision *fft1;
	fftwComplexPrecision *fft2;
	int                  s_size;
};

template <class flccPrecision>
struct makeLccParm {
	int            initialPosition;
	int            offset;
	flccPrecision  *lcc;
	flccPrecision  *sceneTimesBt;
	flccPrecision  *sceneSqTimesBt;
	flccPrecision  *sceneTimesT;
	int            sw, sd;
	int            SW, SD;
	int            t_size;
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Functions prototypes
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
flccResult   fconv2_host               (flccPrecision *,  flccSize,  flccPrecision *,  flccSize,  flccPrecision *,  int,  int,  fftw_plan_t,  fftw_plan_t);
template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
flccResult   fconv3_host               (flccPrecision *,  flccSize,  flccPrecision *,  flccSize,  flccPrecision *,  int,  int,  fftw_plan_t,  fftw_plan_t);
template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
flccResult   fconv2_testStream_host    (flccPrecision *,  flccSize,  fftwComplexPrecision *,  flccSize,  flccPrecision *,  int,  fftw_plan_t,  fftw_plan_t);
template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
flccResult   fconv3_testStream_host    (flccPrecision *,  flccSize,  fftwComplexPrecision *,  flccSize,  flccPrecision *,  int,  fftw_plan_t,  fftw_plan_t);

template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
flccResult   flcorr2_host              (flccPrecision *,  flccSize,  flccPrecision *,  flccSize,  flccPrecision *,  int,  int,  fftw_plan_t,  fftw_plan_t);
template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
flccResult   flcorr3_host              (flccPrecision *,  flccSize,  flccPrecision *,  flccSize,  flccPrecision *,  int,  int,  fftw_plan_t,  fftw_plan_t);
template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
flccResult   flcorr2_testStream_host   (flccPrecision *,  flccSize,  fftwComplexPrecision *,  fftwComplexPrecision *,  flccSize,  flccPrecision *,  int,  fftw_plan_t,  fftw_plan_t);
template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
flccResult   flcorr3_testStream_host   (flccPrecision *,  flccSize,  fftwComplexPrecision *,  fftwComplexPrecision *,  flccSize,  flccPrecision *,  int,  fftw_plan_t,  fftw_plan_t);

template <class flccPrecision, class fftwComplexPrecision>
void         randMatrix_fftw           (fftwComplexPrecision *,  int);

template <class flccPrecision>
void *       padSignal2d               (void *);
template <class flccPrecision>
void *       padSignal3d               (void *);
template <class flccPrecision>
void *       makeConvArray2d           (void *);
template <class flccPrecision>
void *       makeConvArray3d           (void *);
template <class flccPrecision>
void *       makePermanent2d           (void *);
template <class flccPrecision>
void *       makePermanent3d           (void *);
template <class flccPrecision>	
void *       makeTransient2d           (void *);
template <class flccPrecision>
void *       makeTransient3d           (void *);
template <class fftwComplexPrecision>	
void *       complexProduct2d          (void *);
template <class fftwComplexPrecision>
void *       complexProduct3d          (void *);
template <class flccPrecision>	
void *       makeLccArray2d            (void *);
template <class flccPrecision>
void *       makeLccArray3d            (void *);


template <class flccPrecision>
void *fftw_malloc_t (size_t n);

template <class flccPrecision>
void fftw_free_t (void *p);

template <class flccPrecision>
int fftw_init_threads_t ();

template <class flccPrecision>
void fftw_plan_with_nthreads_t (int nthreads);

template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
fftw_plan_t fftw_plan_dft_r2c_2d_t (int n0, int n1, flccPrecision *in, fftwComplexPrecision *out, unsigned flags);

template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
fftw_plan_t fftw_plan_dft_c2r_2d_t (int n0, int n1, fftwComplexPrecision *in, flccPrecision *out, unsigned flags);

template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
fftw_plan_t fftw_plan_dft_r2c_3d_t (int n0, int n1, int n2, flccPrecision *in, fftwComplexPrecision *out, unsigned flags);

template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
fftw_plan_t fftw_plan_dft_c2r_3d_t (int n0, int n1, int n2, fftwComplexPrecision *in, flccPrecision *out, unsigned flags);

template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
void fftw_execute_dft_r2c_t (const fftw_plan_t p, flccPrecision *in, fftwComplexPrecision *out);

template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
void fftw_execute_dft_c2r_t (const fftw_plan_t p, fftwComplexPrecision *in, flccPrecision *out);

template <class fftw_plan_t>
void fftw_destroy_plan_t (fftw_plan_t plan);


#endif // FLCC_FFTW_DEFS_H
