/* 
 * FILE:
 * flcc.h 
 * version 1.0
 *
 * DESCRIPTION:
 * The main include header file for using the FLCC library.
 *
 * AUTHORS:
 * Georgios Papamakarios 
 * Georgios Rizos
 *
 * Aristotle University of Thessaloniki
 * Facutly of Engineering
 * Department of Electrical and Computer Engineering
 *
 * DATE:
 * December 2009
 *
 * CONTACT INFO:
 * e-mail: gpapamak@auth.gr
 *         georgios.rizos12@imperial.ac.uk
 */

 
#ifndef FLCC_H
#define FLCC_H


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Data types definitions
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef enum {
	FLCC_SUCCESS,
	FLCC_INVALID_PLAN,
	FLCC_INVALID_DIMENSION,
	FLCC_INVALID_SIZE,
	FLCC_INVALID_TYPE,
	FLCC_INVALID_PLATFORM,
	FLCC_INVALID_VALUE,
	FLCC_ALLOC_FAILED,
	FLCC_EXEC_FAILED
} flccResult;

typedef struct {
	int h;
	int w;
	int d;
} flccSize;		

typedef enum {
	FLCC_DIRECT2_DEV,
	FLCC_DIRECT3_DEV,
	FLCC_FAST2_DEV,
	FLCC_FAST3_DEV,
	FLCC_DIRECT2_HOST,
	FLCC_DIRECT3_HOST,
	FLCC_FAST2_HOST,
	FLCC_FAST3_HOST,
	FLCC_NULL_METHOD
} flccMethod;

typedef enum {
	FLCC_SINGLE,
	FLCC_STREAM
} flccType;

typedef enum {
	FLCC_HOST,
	FLCC_DEVICE,
	FLCC_ANY
} flccPlatform; 		

typedef struct {
	flccSize        imageSize;
	flccSize        templateSize;
	int             dimension;			
	flccMethod      method;
	flccType        type;
	flccPlatform    platform; 	
	int             numCpuCores;
	int             numGpuDevices;		
	void *          c2r_plan_dev;
	void *          r2c_plan_dev;
	void *          c2r_plan_host_diff;
	void *          r2c_plan_host_diff;
} flccPlan;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Functions prototypes
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef NO_SINGLE_PREC
flccResult    lcorr_plan        (flccPlan *,  int,  flccSize,  flccSize,  flccType,  flccPlatform);
flccResult    lcorr_plan_f      (flccPlan *,  int,  flccSize,  flccSize,  flccType,  flccPlatform);
flccResult    lcorr_exec        (flccPlan,   float *,   float *,   float *,  int);
flccResult    lcorr_exec_f      (flccPlan,   float *,   float *,   float *,  int);
flccResult    lcorr_destroy     (flccPlan *);
flccResult    lcorr_destroy_f   (flccPlan *);

flccResult    conv_plan         (flccPlan *,  int,  flccSize,  flccSize,  flccType,  flccPlatform);
flccResult    conv_plan_f       (flccPlan *,  int,  flccSize,  flccSize,  flccType,  flccPlatform);
flccResult    conv_exec         (flccPlan,   float *,   float *,   float *,  int);
flccResult    conv_exec_f       (flccPlan,   float *,   float *,   float *,  int);
flccResult    conv_destroy      (flccPlan *);
flccResult    conv_destroy_f    (flccPlan *);
#endif

#ifndef NO_DOUBLE_PREC
flccResult    lcorr_plan_d      (flccPlan *,  int,  flccSize,  flccSize,  flccType,  flccPlatform);
flccResult    lcorr_exec_d      (flccPlan,  double *,  double *,  double *,  int);
flccResult    lcorr_destroy_d   (flccPlan *);

flccResult    conv_plan_d       (flccPlan *,  int,  flccSize,  flccSize,  flccType,  flccPlatform);
flccResult    conv_exec_d       (flccPlan,  double *,  double *,  double *,  int);
flccResult    conv_destroy_d    (flccPlan *);
#endif

void *        flcc_malloc       (size_t);
void          flcc_free         (void *);


#endif // FLCC_H

