/* 
 * FILE:
 * gen_direct2_thread.h 
 * version 1.0
 *
 * DESCRIPTION:
 * It includes macro "function-generators" which generate
 * the optimized CPU threads for the 2d direct method.
 *
 * AUTHORS:
 * Georgios Papamakarios 
 * Georgios Rizos
 *
 * Aristotle University of Thessaloniki
 * Facutly of Engineering
 * Department of Electrical and Computer Engineering
 *
 * DATE:
 * April 2011
 *
 * CONTACT INFO:
 * e-mail: gpapamak@auth.gr
 *         georgios.rizos12@imperial.ac.uk
 */


/////////////////
#define GEN_DIRECT2_THREAD_PROTOTYPE( FUNCTION_NAME, TH, TW )				\
template <class flccPrecision>												\
void *FUNCTION_NAME ## _ ## TH ## x ## TW (void *arg)

/////////////////
#define GEN_DIRECT2_THREAD_INSTANCE( FUNCTION_NAME, TH, TW, type )			\
template 																	\
void *FUNCTION_NAME ## _ ## TH ## x ## TW <type>		 					\
(void *arg);
		
/////////////////	
#define GEN_DIRECT2_THREAD_CONV( TH, TW ) 									\
GEN_DIRECT2_THREAD_PROTOTYPE( convolution2d_thread, TH, TW )				\
{																			\
	const corrParm<flccPrecision> *c = (corrParm<flccPrecision> *)arg;		\
	for (int n=0; n<c->offset; n++) {										\
		const int indx = c->initialPosition + n;							\
		const int u = indx%c->sw;											\
		const int v = indx/c->sw;											\
		register flccPrecision conv = 0.0;									\
		int t_indx = (TH)*(TW)-1;											\
		int i_indx = v*c->bw + u;											\
		for (int j=0; j<(TH); j++) {										\
			for (int i=0; i<(TW); i++) {									\
				conv += c->image[i_indx+i] * c->templat[t_indx-i]; 			\
			}																\
			i_indx += c->bw;												\
			t_indx -= (TW);													\
		}																	\
		c->lcc[indx] = conv; }												\
	pthread_exit (NULL);													\
}

/////////////////
#define GEN_DIRECT2_THREAD_CONV_INSTANCE( TH, TW, type )					\
GEN_DIRECT2_THREAD_INSTANCE( convolution2d_thread, TH, TW, type )

/////////////////
#define GEN_DIRECT2_THREAD_LCORR( TH, TW ) 									\
GEN_DIRECT2_THREAD_PROTOTYPE( lcorrelation2d_thread, TH, TW )				\
{																			\
	const corrParm<flccPrecision> *c = (corrParm<flccPrecision> *)arg;		\
	for (int n=0; n<c->offset; n++) {										\
		const int indx = c->initialPosition + n;							\
		const int u = indx%c->sw;											\
		const int v = indx/c->sw;											\
		register flccPrecision corr   = 0.0;								\
		register flccPrecision sum    = 0.0;								\
		register flccPrecision sum_sq = 0.0;								\
		int t_indx = 0;														\
		int i_indx = v*c->bw + u;											\
		for (int j=0; j<(TH); j++) {										\
			for (int i=0; i<(TW); i++) {									\
				const flccPrecision im = c->image[i_indx+i];				\
				corr   += im * c->templat[t_indx+i];						\
				sum    += im;												\
				sum_sq += im * im; 											\
			} 																\
			i_indx += c->bw;												\
			t_indx += (TW);													\
		}																	\
		const flccPrecision var = sum_sq - sum*sum/((TH)*(TW));				\
		if (var == 0)	c->lcc[indx] = 0.0;									\
		else			c->lcc[indx] = corr / sqrt(var); } 					\
	pthread_exit (NULL);													\
}

/////////////////
#define GEN_DIRECT2_THREAD_LCORR_INSTANCE( TH, TW, type )					\
GEN_DIRECT2_THREAD_INSTANCE( lcorrelation2d_thread, TH, TW, type )

