/* 
 * FILE:
 * gen_direct3_thread.h 
 * version 1.0
 *
 * DESCRIPTION:
 * It includes macro "function-generators" which generate
 * the optimized CPU threads for the 3d direct method.
 *
 * AUTHORS:
 * Georgios Papamakarios 
 * Georgios Rizos
 *
 * Aristotle University of Thessaloniki
 * Facutly of Engineering
 * Department of Electrical and Computer Engineering
 *
 * DATE:
 * April 2011
 *
 * CONTACT INFO:
 * e-mail: gpapamak@auth.gr
 *         georgios.rizos12@imperial.ac.uk
 */


/////////////////
#define GEN_DIRECT3_THREAD_PROTOTYPE( FUNCTION_NAME, TH, TW, TD )			\
template <class flccPrecision>												\
void *FUNCTION_NAME ## _ ## TH ## x ## TW ## x ## TD (void *arg)

/////////////////
#define GEN_DIRECT3_THREAD_INSTANCE( FUNCTION_NAME, TH, TW, TD, type )		\
template 																	\
void *FUNCTION_NAME ## _ ## TH ## x ## TW ## x ## TD <type>			 		\
(void *arg);
		
/////////////////	
#define GEN_DIRECT3_THREAD_CONV( TH, TW, TD ) 								\
GEN_DIRECT3_THREAD_PROTOTYPE( convolution3d_thread, TH, TW, TD )			\
{																			\
	const corrParm<flccPrecision> *c = (corrParm<flccPrecision> *)arg;		\
	for (int n=0; n<c->offset; n++) {										\
		const int indx = c->initialPosition + n;							\
		const int u = (indx%(c->sd*c->sw))%c->sd;							\
		const int v = (indx%(c->sd*c->sw))/c->sd;							\
		const int w = indx/(c->sd*c->sw);									\
		register flccPrecision conv = 0.0;									\
		int t_indx = (TH)*(TW)*(TD)-1;										\
		int i_indx = w*c->bd*c->bw + v*c->bd + u;							\
		for (int k=0; k<(TH); k++) {										\
			for (int j=0; j<(TW); j++) {									\
				for (int i=0; i<(TD); i++) {								\
					conv += c->image[i_indx+i] * c->templat[t_indx-i]; 		\
				}															\
				i_indx += c->bd;											\
				t_indx -= (TD);												\
			}																\
			i_indx += c->bd*(c->bw-(TD));									\
		}																	\
		c->lcc[indx] = conv; }												\
	pthread_exit (NULL);													\
}

/////////////////
#define GEN_DIRECT3_THREAD_CONV_INSTANCE( TH, TW, TD, type )				\
GEN_DIRECT3_THREAD_INSTANCE( convolution3d_thread, TH, TW, TD, type )				

/////////////////
#define GEN_DIRECT3_THREAD_LCORR( TH, TW, TD )								\
GEN_DIRECT3_THREAD_PROTOTYPE( lcorrelation3d_thread, TH, TW, TD )			\
{																			\
	const corrParm<flccPrecision> *c = (corrParm<flccPrecision> *)arg;		\
	for (int n=0; n<c->offset; n++) {										\
		const int indx = c->initialPosition + n;							\
		const int u = (indx%(c->sd*c->sw))%c->sd;							\
		const int v = (indx%(c->sd*c->sw))/c->sd;							\
		const int w = indx/(c->sd*c->sw);									\
		register flccPrecision corr   = 0.0;								\
		register flccPrecision sum    = 0.0;								\
		register flccPrecision sum_sq = 0.0;								\
		int t_indx = 0;														\
		int i_indx = w*c->bd*c->bw + v*c->bd + u;							\
		for (int k=0; k<(TH); k++) {										\
			for (int j=0; j<(TW); j++) {									\
				for (int i=0; i<(TD); i++) {								\
					const flccPrecision im = c->image[i_indx+i];			\
					corr   += im * c->templat[t_indx+i];					\
					sum    += im;											\
					sum_sq += im * im; 										\
				} 															\
				i_indx += c->bd;											\
				t_indx += (TD);												\
			}																\
			i_indx += c->bd*(c->bw-(TD));									\
		}																	\
		const flccPrecision var = sum_sq - sum*sum/((TH)*(TW)*(TD));		\
		if (var == 0)	c->lcc[indx] = 0.0;									\
		else			c->lcc[indx] = corr / sqrt(var); } 					\
	pthread_exit (NULL);													\
}

/////////////////
#define GEN_DIRECT3_THREAD_LCORR_INSTANCE( TH, TW, TD, type )				\
GEN_DIRECT3_THREAD_INSTANCE( lcorrelation3d_thread, TH, TW, TD, type )	

