/* 
 * FILE:
 * conv_destroy.c
 * version 1.1
 *
 * DESCRIPTION:
 * Matlab mex function for conv_destroy.
 *
 * AUTHORS:
 * Georgios Papamakarios 
 *
 * Aristotle University of Thessaloniki
 * Facutly of Engineering
 * Department of Electrical and Computer Engineering
 *
 * DATE:
 * January 2013
 *
 * CONTACT INFO:
 * e-mail: gpapamak@auth.gr
 */

#include "mex.h"
#include "matrix.h"
#include <flcc.h>
#include <string.h>
#include <stdlib.h>

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
	if (nrhs<1 || nrhs>2) {mexErrMsgTxt("Incorrect number of inputs.");}
	if (nlhs!=0)          {mexErrMsgTxt("Incorrect number of outputs.");}
	
	// get FLCC plan
	flccPlan **plan = (flccPlan **)mxGetData(prhs[0]);
	
	// destroy the plan
	flccResult rc;
	if (nrhs<2) {
		#ifndef NO_SINGLE_PREC
		rc = conv_destroy(*plan);
		#else
		mexErrMsgTxt("Single precision is not installed.");
		#endif
	}
	else {
		char bufStr[7];
		mxGetString(prhs[1], bufStr, 7);
		if (!strcmp(bufStr, "single")) {
			#ifndef NO_SINGLE_PREC
			rc = conv_destroy_f(*plan);
			#else
			mexErrMsgTxt("Single precision is not installed.");
			#endif
		}
		else if (!strcmp(bufStr, "double")) {
			#ifndef NO_DOUBLE_PREC
			rc = conv_destroy_d(*plan);
			#else
			mexErrMsgTxt("Double precision is not installed.");
			#endif
		}
		else {mexErrMsgTxt("Incorrect precision.");}
	}
	switch (rc)	{
	case FLCC_SUCCESS:
		break;
	case FLCC_INVALID_VALUE:
		mexErrMsgTxt("Invalid plan.");
		break;
	default:
		mexErrMsgTxt("FLCC plan destruction failed.");
	}
	
	// free plan
	free(*plan);
}
