function r = normxcorr3(t,x)
% r = normxcorr3(t,x)
% 3D LCC in MATLAB.
% 3D generalization of normxcorr2.
% We will use this until MATLAB implements normxcorr3.
%
% INPUTS:
%     t    3D template
%     x    3D image
%
% OUTPUTS:
%     r    3D LCC
%
% Georgios Papamakarios (gpapamak@auth.gr)
% Electrical and Computer Engineering AUTH
% September 2012


Sx = size(x);
St = size(t);
Nt = numel(t);

t = (t - mean(t(:))) / (sqrt(Nt) * std(t(:),1));
t = t(end:-1:1,end:-1:1,end:-1:1);

if Nt < 8.75*log2(numel(x))
	xt = convn(x,t);
	x = padarray(x,St-1,0,'post');
else
	x = padarray(x,St-1,0,'post');
	t = padarray(t,Sx-1,0,'post');
	xt = ifftn(fftn(x).*fftn(t));       
end

x = padarray(x,St,0,'pre');
x2 = x.^2;

x = cumsum(x,1);
x = x(St(1)+1:end,:,:) - x(1:end-St(1),:,:);
x = cumsum(x,2);
x = x(:,St(2)+1:end,:) - x(:,1:end-St(2),:);
x = cumsum(x,3);
x = x(:,:,St(3)+1:end) - x(:,:,1:end-St(3));

x2 = cumsum(x2,1);
x2 = x2(St(1)+1:end,:,:) - x2(1:end-St(1),:,:);
x2 = cumsum(x2,2);
x2 = x2(:,St(2)+1:end,:) - x2(:,1:end-St(2),:);
x2 = cumsum(x2,3);
x2 = x2(:,:,St(3)+1:end) - x2(:,:,1:end-St(3));

r = xt ./ sqrt(x2 - x.*x/Nt);
