/* 
 * FILE:
 * conv_destroy.c
 * version 1.1
 *
 * DESCRIPTION:
 * This function calls the non-API template function that frees the resources occupied by a convolution plan.
 *
 * AUTHORS:
 * Georgios Papamakarios 
 * Georgios Rizos
 *
 * Aristotle University of Thessaloniki
 * Faculty of Engineering
 * Department of Electrical and Computer Engineering
 *
 * DATE:
 * April 2011
 *
 * CONTACT INFO:
 * e-mail: gpapamak@auth.gr
 *         georgios.rizos12@imperial.ac.uk
 */


// System include
#include <FLCC_general_defs.h>
#ifndef NO_FFTW
#include <FLCC_fftw_defs.h>
#endif


// Deallocation of memory

#ifndef NO_SINGLE_PREC
flccResult conv_destroy_f (flccPlan *plan)
{
	if (plan == NULL)
		return FLCC_INVALID_VALUE;
	
	#ifndef NO_FFTW
	return conv_destroy_t<float, fftwf_plan> (plan);
	#else
	return conv_destroy_t<float> (plan);
	#endif
}

flccResult conv_destroy (flccPlan *plan)
{
	return conv_destroy_f(plan);
}
#endif

#ifndef NO_DOUBLE_PREC
flccResult conv_destroy_d (flccPlan *plan)
{
	if (plan == NULL)
		return FLCC_INVALID_VALUE;
    
	#ifndef NO_FFTW
	return conv_destroy_t<double, fftw_plan> (plan);
	#else
	return conv_destroy_t<double> (plan);
	#endif
}
#endif
