/* 
 * FILE:
 * conv_plan.c
 * version 1.2
 *
 * DESCRIPTION:
 * This function calls the non-API template function that selects among the available routines for convolution computation
 * according to the size and dimension specified.
 *
 * AUTHORS:
 * Georgios Papamakarios 
 * Georgios Rizos
 *
 * Aristotle University of Thessaloniki
 * Faculty of Engineering
 * Department of Electrical and Computer Engineering
 *
 * DATE:
 * November 2011
 *
 * CONTACT INFO:
 * e-mail: gpapamak@auth.gr
 *         georgios.rizos12@imperial.ac.uk
 */

 
// System include
#include <FLCC_general_defs.h>
#ifndef NO_CUDA	
#include <FLCC_cuda_defs.h>
#endif
#ifndef NO_FFTW
#include <FLCC_fftw_defs.h>
#endif


// Set plan parameters to be used in convolution computation

#ifndef NO_SINGLE_PREC
flccResult conv_plan (flccPlan *plan, int dim, flccSize imSize, flccSize temSize, flccType type, flccPlatform platform)
{
	return conv_plan_f(plan, dim, imSize, temSize, type, platform);
}

flccResult conv_plan_f (flccPlan *plan, int dim, flccSize imSize, flccSize temSize, flccType type, flccPlatform platform)
{
	#ifndef NO_FFTW
		#ifndef NO_CUDA	
			return conv_plan_t<float, fftwf_complex, fftwf_plan, cufftReal, cufftComplex> (plan, dim, imSize, temSize, type, platform);
		#else
			return conv_plan_t<float, fftwf_complex, fftwf_plan> (plan, dim, imSize, temSize, type, platform);
		#endif
	#else
		#ifndef NO_CUDA
			return conv_plan_t<float, cufftReal, cufftComplex> (plan, dim, imSize, temSize, type, platform);
		#else
			return conv_plan_t<float> (plan, dim, imSize, temSize, type, platform);
		#endif
	#endif
}
#endif

#ifndef NO_DOUBLE_PREC
flccResult conv_plan_d (flccPlan *plan, int dim, flccSize imSize, flccSize temSize, flccType type, flccPlatform platform)
{
	#ifndef NO_FFTW
		#ifndef NO_CUDA	
			return conv_plan_t<double, fftw_complex, fftw_plan, cufftDoubleReal, cufftDoubleComplex> (plan, dim, imSize, temSize, type, platform);
		#else
			return conv_plan_t<double, fftw_complex, fftw_plan> (plan, dim, imSize, temSize, type, platform);
		#endif
	#else
		#ifndef NO_CUDA
			return conv_plan_t<double, cufftDoubleReal, cufftDoubleComplex> (plan, dim, imSize, temSize, type, platform);
		#else
			return conv_plan_t<double> (plan, dim, imSize, temSize, type, platform);
		#endif
	#endif
}
#endif
