/* 
 * FILE:
 * flcc_malloc.c
 * version 1.1
 *
 * DESCRIPTION:
 * This function allocates appropriately CPU memory for usage in the FLCC library routines.
 *
 * AUTHORS:
 * Georgios Papamakarios 
 * Georgios Rizos
 *
 * Aristotle University of Thessaloniki
 * Facutly of Engineering
 * Department of Electrical and Computer Engineering
 *
 * DATE:
 * April 2011
 *
 * CONTACT INFO:
 * e-mail: gpapamak@auth.gr
 *         grizos@auth.gr
 */
 
 
#include <FLCC_general_defs.h>
#ifndef NO_CUDA	
#include <FLCC_cuda_defs.h>
#endif
 
 
void *flcc_malloc (size_t bytes)
{
	void *array;

	#ifndef NO_CUDA	
	
	cudaError_t err = cudaHostAlloc ((void **)&array, bytes, cudaHostAllocDefault);
	
	if (err != cudaSuccess)
		return NULL;
		
	#else
	
	array = malloc (bytes);
	
	#endif

	return array;
}
