/* 
 * FILE:
 * lcorr_exec.c
 * version 1.1
 *
 * DESCRIPTION:
 * This function calls the non-API template function that computes the LCC distribution of an image with a template according to the plan specified.
 *
 * AUTHORS:
 * Georgios Papamakarios 
 * Georgios Rizos
 *
 * Aristotle University of Thessaloniki
 * Faculty of Engineering
 * Department of Electrical and Computer Engineering
 *
 * DATE:
 * April 2011
 *
 * CONTACT INFO:
 * e-mail: gpapamak@auth.gr
 *         georgios.rizos12@imperial.ac.uk
 */

 
// System include
#include <FLCC_general_defs.h>
#ifndef NO_CUDA	
#include <FLCC_cuda_defs.h>
#endif
#ifndef NO_FFTW
#include <FLCC_fftw_defs.h>
#endif


// Execution of LCC problem using a pre-made plan

#ifndef NO_SINGLE_PREC
flccResult lcorr_exec (flccPlan plan, float *image, float *templat, float *lcc, int imCnt)
{
	return lcorr_exec_f(plan, image, templat, lcc, imCnt);
}

flccResult lcorr_exec_f (flccPlan plan, float *image, float *templat, float *lcc, int imCnt)
{
	#ifndef NO_FFTW
		#ifndef NO_CUDA	
			return lcorr_exec_t<float, fftwf_complex, fftwf_plan, cufftReal, cufftComplex> (plan, image, templat, lcc, imCnt);
		#else
			return lcorr_exec_t<float, fftwf_complex, fftwf_plan> (plan, image, templat, lcc, imCnt);
		#endif
	#else
		#ifndef NO_CUDA
			return lcorr_exec_t<float, cufftReal, cufftComplex> (plan, image, templat, lcc, imCnt);
		#else
			return lcorr_exec_t<float> (plan, image, templat, lcc, imCnt);
		#endif
	#endif
}
#endif

#ifndef NO_DOUBLE_PREC
flccResult lcorr_exec_d (flccPlan plan, double *image, double *templat, double *lcc, int imCnt)
{
	#ifndef NO_FFTW
		#ifndef NO_CUDA	
			return lcorr_exec_t<double, fftw_complex, fftw_plan, cufftDoubleReal, cufftDoubleComplex> (plan, image, templat, lcc, imCnt);
		#else
			return lcorr_exec_t<double, fftw_complex, fftw_plan> (plan, image, templat, lcc, imCnt);
		#endif
	#else
		#ifndef NO_CUDA
			return lcorr_exec_t<double, cufftDoubleReal, cufftDoubleComplex> (plan, image, templat, lcc, imCnt);
		#else
			return lcorr_exec_t<double> (plan, image, templat, lcc, imCnt);
		#endif
	#endif
}
#endif
