/* 
 * FILE:
 * conv_destroy_t.c
 * version 1.5
 *
 * DESCRIPTION:
 * This non-API template function frees the resources occupied by a convolution plan.
 *
 * AUTHORS:
 * Georgios Papamakarios 
 * Georgios Rizos
 *
 * Aristotle University of Thessaloniki
 * Faculty of Engineering
 * Department of Electrical and Computer Engineering
 *
 * DATE:
 * April 2011
 *
 * CONTACT INFO:
 * e-mail: gpapamak@auth.gr
 *         georgios.rizos12@imperial.ac.uk
 */


// System include
#include <FLCC_general_defs.h>
#ifndef NO_CUDA	
#include <FLCC_cuda_defs.h>
#endif
#ifndef NO_FFTW
#include <FLCC_fftw_defs.h>
#endif

// Template function for the deallocation of memory
#ifndef NO_FFTW
template <class flccPrecision, class fftw_plan_t>
#else
template <class flccPrecision>
#endif
flccResult conv_destroy_t (flccPlan *plan)
{
	#ifndef NO_CUDA		
	if (plan->method==FLCC_FAST2_DEV || plan->method==FLCC_FAST3_DEV)
	{
		cufftHandle *c2r_plan_dev = (cufftHandle *)plan->c2r_plan_dev;
		cufftHandle *r2c_plan_dev = (cufftHandle *)plan->r2c_plan_dev;
	
		CUFFT_SAFE_CALL ( cufftDestroy (*c2r_plan_dev) );
		CUFFT_SAFE_CALL ( cufftDestroy (*r2c_plan_dev) );
		
		free (c2r_plan_dev);
		free (r2c_plan_dev);
	}
	#endif

	#ifndef NO_FFTW
	if (plan->method==FLCC_FAST2_HOST || plan->method==FLCC_FAST3_HOST)
	{
		fftw_plan_t *c2r_plan_host_diff = (fftw_plan_t *)plan->c2r_plan_host_diff;
		fftw_plan_t *r2c_plan_host_diff = (fftw_plan_t *)plan->r2c_plan_host_diff;
		
		fftw_destroy_plan_t<fftw_plan_t> (*c2r_plan_host_diff);
		fftw_destroy_plan_t<fftw_plan_t> (*r2c_plan_host_diff);
		
		free (c2r_plan_host_diff);
		free (r2c_plan_host_diff);
	}
	#endif
	
	return FLCC_SUCCESS;
}


// make instants
#ifndef NO_FFTW
	#ifndef NO_SINGLE_PREC
	template flccResult conv_destroy_t<float,  fftwf_plan> (flccPlan *plan);
	#endif
	#ifndef NO_DOUBLE_PREC
	template flccResult conv_destroy_t<double, fftw_plan>  (flccPlan *plan);
	#endif
#else
	#ifndef NO_SINGLE_PREC
	template flccResult conv_destroy_t<float>  (flccPlan *plan);
	#endif
	#ifndef NO_DOUBLE_PREC
	template flccResult conv_destroy_t<double> (flccPlan *plan);
	#endif
#endif
