/* 
 * FILE:
 * FLCC_cufft_wrappers.cu
 * version 1.0
 *
 * DESCRIPTION:
 * These are the template wrappers for CUFFT.
 *
 * AUTHORS:
 * Georgios Papamakarios 
 * Georgios Rizos
 *
 * Aristotle University of Thessaloniki
 * Facutly of Engineering
 * Department of Electrical and Computer Engineering
 *
 * DATE:
 * February 2013
 *
 * CONTACT INFO:
 * e-mail: gpapamak@auth.gr
 *         georgios.rizos12@imperial.ac.uk
 */

 
#include <FLCC_general_defs.h>
#include <FLCC_cuda_defs.h>



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Wrappers for 2D FFT planning
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class flccPrecision>
cufftResult cufftPlan2d_R2C_t (cufftHandle *plan, int nx, int ny) 
{
	return CUFFT_EXEC_FAILED;
}

#ifndef NO_SINGLE_PREC
template <>
cufftResult cufftPlan2d_R2C_t<float> (cufftHandle *plan, int nx, int ny)
{
	return cufftPlan2d(plan, nx, ny, CUFFT_R2C);
}
#endif

#ifndef NO_DOUBLE_PREC
template <>
cufftResult cufftPlan2d_R2C_t<double> (cufftHandle *plan, int nx, int ny)
{
	return cufftPlan2d(plan, nx, ny, CUFFT_D2Z);
}
#endif

template <class flccPrecision>
cufftResult cufftPlan2d_C2R_t (cufftHandle *plan, int nx, int ny) 
{
	return CUFFT_EXEC_FAILED;
}

#ifndef NO_SINGLE_PREC
template <>
cufftResult cufftPlan2d_C2R_t<float> (cufftHandle *plan, int nx, int ny)
{
	return cufftPlan2d(plan, nx, ny, CUFFT_C2R);
}
#endif

#ifndef NO_DOUBLE_PREC
template <>
cufftResult cufftPlan2d_C2R_t<double> (cufftHandle *plan, int nx, int ny)
{
	return cufftPlan2d(plan, nx, ny, CUFFT_Z2D);
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Wrappers for 3D FFT planning
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class flccPrecision>
cufftResult cufftPlan3d_R2C_t (cufftHandle *plan, int nx, int ny, int nz) 
{
	return CUFFT_EXEC_FAILED;
}

#ifndef NO_SINGLE_PREC
template <>
cufftResult cufftPlan3d_R2C_t<float> (cufftHandle *plan, int nx, int ny, int nz)
{
	return cufftPlan3d(plan, nx, ny, nz, CUFFT_R2C);
}
#endif

#ifndef NO_DOUBLE_PREC
template <>
cufftResult cufftPlan3d_R2C_t<double> (cufftHandle *plan, int nx, int ny, int nz)
{
	return cufftPlan3d(plan, nx, ny, nz, CUFFT_D2Z);
}
#endif

template <class flccPrecision>
cufftResult cufftPlan3d_C2R_t (cufftHandle *plan, int nx, int ny, int nz) 
{
	return CUFFT_EXEC_FAILED;
}

#ifndef NO_SINGLE_PREC
template <>
cufftResult cufftPlan3d_C2R_t<float> (cufftHandle *plan, int nx, int ny, int nz)
{
	return cufftPlan3d(plan, nx, ny, nz, CUFFT_C2R);
}
#endif

#ifndef NO_DOUBLE_PREC
template <>
cufftResult cufftPlan3d_C2R_t<double> (cufftHandle *plan, int nx, int ny, int nz)
{
	return cufftPlan3d(plan, nx, ny, nz, CUFFT_Z2D);
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Wrappers for real to complex FFT execution
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class cufftRealPrecision, class cufftComplexPrecision>
cufftResult cufftExecR2C_t (cufftHandle plan, cufftRealPrecision *idata, cufftComplexPrecision *odata) 
{
	return CUFFT_EXEC_FAILED;
}

#ifndef NO_SINGLE_PREC
template <>
cufftResult cufftExecR2C_t<cufftReal, cufftComplex> (cufftHandle plan, cufftReal *idata, cufftComplex *odata)
{
	return cufftExecR2C(plan, idata, odata);
}
#endif

#ifndef NO_DOUBLE_PREC
template <>
cufftResult cufftExecR2C_t<cufftDoubleReal, cufftDoubleComplex> (cufftHandle plan, cufftDoubleReal *idata, cufftDoubleComplex *odata)
{
	return cufftExecD2Z(plan, idata, odata);
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Wrappers for complex to real FFT execution
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class cufftRealPrecision, class cufftComplexPrecision>
cufftResult cufftExecC2R_t (cufftHandle plan, cufftComplexPrecision *idata, cufftRealPrecision *odata) 
{
	return CUFFT_EXEC_FAILED;
}

#ifndef NO_SINGLE_PREC
template <>
cufftResult cufftExecC2R_t<cufftReal, cufftComplex> (cufftHandle plan, cufftComplex *idata, cufftReal *odata)
{
	return cufftExecC2R(plan, idata, odata);
}
#endif

#ifndef NO_DOUBLE_PREC
template <>
cufftResult cufftExecC2R_t<cufftDoubleReal, cufftDoubleComplex> (cufftHandle plan, cufftDoubleComplex *idata, cufftDoubleReal *odata)
{
	return cufftExecZ2D(plan, idata, odata);
}
#endif
