/* 
 * FILE:
 * FLCC_fftw_wrappers.c
 * version 1.0
 *
 * DESCRIPTION:
 * These are the template wrappers for FFTW.
 *
 * AUTHORS:
 * Georgios Papamakarios 
 * Georgios Rizos
 *
 * Aristotle University of Thessaloniki
 * Facutly of Engineering
 * Department of Electrical and Computer Engineering
 *
 * DATE:
 * February 2013
 *
 * CONTACT INFO:
 * e-mail: gpapamak@auth.gr
 *         georgios.rizos12@imperial.ac.uk
 */
 

#include <FLCC_general_defs.h>
#include <FLCC_fftw_defs.h>


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Wrappers for memory allocation
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class flccPrecision>
void *fftw_malloc_t (size_t n) 
{
	return;
}

#ifndef NO_SINGLE_PREC
template <>
void *fftw_malloc_t<float> (size_t n)
{
	return fftwf_malloc(n);
}
#endif

#ifndef NO_DOUBLE_PREC
template <>
void *fftw_malloc_t<double> (size_t n)
{
	return fftw_malloc(n);
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Wrappers for memory deallocation
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class flccPrecision>
void fftw_free_t (void *p) 
{
	return;
}

#ifndef NO_SINGLE_PREC
template <>
void fftw_free_t<float> (void *p)
{
	fftwf_free(p);
}
#endif

#ifndef NO_DOUBLE_PREC
template <>
void fftw_free_t<double> (void *p)
{
	fftw_free(p);
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Wrappers for threads initialization
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class flccPrecision>
int fftw_init_threads_t () 
{
	return 0;
}

#ifndef NO_SINGLE_PREC
template <>
int fftw_init_threads_t<float> ()
{
	return fftwf_init_threads();
}
#endif

#ifndef NO_DOUBLE_PREC
template <>
int fftw_init_threads_t<double> ()
{
	return fftw_init_threads();
}
#endif

template <class flccPrecision>
void fftw_plan_with_nthreads_t (int nthreads) 
{
	return;
}

#ifndef NO_SINGLE_PREC
template <>
void fftw_plan_with_nthreads_t<float> (int nthreads)
{
	fftwf_plan_with_nthreads(nthreads);
}
#endif

#ifndef NO_DOUBLE_PREC
template <>
void fftw_plan_with_nthreads_t<double> (int nthreads)
{
	fftw_plan_with_nthreads(nthreads);
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Wrappers for 2D FFT planning
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
fftw_plan_t fftw_plan_dft_r2c_2d_t (int n0, int n1, flccPrecision *in, fftwComplexPrecision *out, unsigned flags) 
{
	return NULL;
}

#ifndef NO_SINGLE_PREC
template <>
fftwf_plan fftw_plan_dft_r2c_2d_t<float, fftwf_plan, fftwf_complex> (int n0, int n1, float *in, fftwf_complex *out, unsigned flags)
{
	return fftwf_plan_dft_r2c_2d(n0, n1, in, out, flags);
}
#endif

#ifndef NO_DOUBLE_PREC
template <>
fftw_plan fftw_plan_dft_r2c_2d_t<double, fftw_plan, fftw_complex> (int n0, int n1, double *in, fftw_complex *out, unsigned flags)
{
	return fftw_plan_dft_r2c_2d(n0, n1, in, out, flags);
}
#endif

template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
fftw_plan_t fftw_plan_dft_c2r_2d_t (int n0, int n1, fftwComplexPrecision *in, flccPrecision *out, unsigned flags) 
{
	return NULL;
}

#ifndef NO_SINGLE_PREC
template <>
fftwf_plan fftw_plan_dft_c2r_2d_t<float, fftwf_plan, fftwf_complex> (int n0, int n1, fftwf_complex *in, float *out, unsigned flags)
{
	return fftwf_plan_dft_c2r_2d(n0, n1, in, out, flags);
}
#endif

#ifndef NO_DOUBLE_PREC
template <>
fftw_plan fftw_plan_dft_c2r_2d_t<double, fftw_plan, fftw_complex> (int n0, int n1, fftw_complex *in, double *out, unsigned flags)
{
	return fftw_plan_dft_c2r_2d(n0, n1, in, out, flags);
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Wrappers for 3D FFT planning
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
fftw_plan_t fftw_plan_dft_r2c_3d_t (int n0, int n1, int n2, flccPrecision *in, fftwComplexPrecision *out, unsigned flags) 
{
	return NULL;
}

#ifndef NO_SINGLE_PREC
template <>
fftwf_plan fftw_plan_dft_r2c_3d_t<float, fftwf_plan, fftwf_complex> (int n0, int n1, int n2, float *in, fftwf_complex *out, unsigned flags)
{
	return fftwf_plan_dft_r2c_3d(n0, n1, n2, in, out, flags);
}
#endif

#ifndef NO_DOUBLE_PREC
template <>
fftw_plan fftw_plan_dft_r2c_3d_t<double, fftw_plan, fftw_complex> (int n0, int n1, int n2, double *in, fftw_complex *out, unsigned flags)
{
	return fftw_plan_dft_r2c_3d(n0, n1, n2, in, out, flags);
}
#endif

template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
fftw_plan_t fftw_plan_dft_c2r_3d_t (int n0, int n1, int n2, fftwComplexPrecision *in, flccPrecision *out, unsigned flags) 
{
	return NULL;
}

#ifndef NO_SINGLE_PREC
template <>
fftwf_plan fftw_plan_dft_c2r_3d_t<float, fftwf_plan, fftwf_complex> (int n0, int n1, int n2, fftwf_complex *in, float *out, unsigned flags)
{
	return fftwf_plan_dft_c2r_3d(n0, n1, n2, in, out, flags);
}
#endif

#ifndef NO_DOUBLE_PREC
template <>
fftw_plan fftw_plan_dft_c2r_3d_t<double, fftw_plan, fftw_complex> (int n0, int n1, int n2, fftw_complex *in, double *out, unsigned flags)
{
	return fftw_plan_dft_c2r_3d(n0, n1, n2, in, out, flags);
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Wrappers for real to complex FFT execution
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
void fftw_execute_dft_r2c_t (const fftw_plan_t p, flccPrecision *in, fftwComplexPrecision *out) 
{
	return;
}

#ifndef NO_SINGLE_PREC
template <>
void fftw_execute_dft_r2c_t<float, fftwf_plan, fftwf_complex> (const fftwf_plan p, float *in, fftwf_complex *out)
{
	return fftwf_execute_dft_r2c(p, in, out);
}
#endif

#ifndef NO_DOUBLE_PREC
template <>
void fftw_execute_dft_r2c_t<double, fftw_plan, fftw_complex> (const fftw_plan p, double *in, fftw_complex *out)
{
	return fftw_execute_dft_r2c(p, in, out);
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Wrappers for complex to real FFT execution
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class flccPrecision, class fftw_plan_t, class fftwComplexPrecision>
void fftw_execute_dft_c2r_t (const fftw_plan_t p, fftwComplexPrecision *in, flccPrecision *out) 
{
	return;
}

#ifndef NO_SINGLE_PREC
template <>
void fftw_execute_dft_c2r_t<float, fftwf_plan, fftwf_complex> (const fftwf_plan p, fftwf_complex *in, float *out)
{
	return fftwf_execute_dft_c2r(p, in, out);
}
#endif

#ifndef NO_DOUBLE_PREC
template <>
void fftw_execute_dft_c2r_t<double, fftw_plan, fftw_complex> (const fftw_plan p, fftw_complex *in, double *out)
{
	return fftw_execute_dft_c2r(p, in, out);
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Wrappers for plan destruction
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class fftw_plan_t>
void fftw_destroy_plan_t (fftw_plan_t plan) 
{
	return;
}

#ifndef NO_SINGLE_PREC
template <>
void fftw_destroy_plan_t<fftwf_plan> (fftwf_plan plan)
{
	fftwf_destroy_plan(plan);
}
#endif

#ifndef NO_DOUBLE_PREC
template <>
void fftw_destroy_plan_t<fftw_plan> (fftw_plan plan)
{
	fftw_destroy_plan(plan);
}
#endif
