/* 
 * FILE:
 * conv_select.c
 * version 1.0
 *
 * DESCRIPTION:
 * This program checks which convolution method is selected by the FLCC library planning routine.
 *
 * AUTHORS:
 * Georgios Papamakarios 
 * Georgios Rizos
 *
 * Aristotle University of Thessaloniki
 * Facutly of Engineering
 * Department of Electrical and Computer Engineering
 *
 * DATE:
 * October 2011
 *
 * CONTACT INFO:
 * e-mail: gpapamak@auth.gr
 *         georgios.rizos12@imperial.ac.uk
 */

 
#include <iostream>
#include <stdlib.h>
#include <flcc.h>
using namespace std;


int main (int argc, char **argv)
{	
	if (argc!=7 && argc!=9)
	{
		cout << "USAGE:\t" << argv[0] << " IH IW TH TW flccType flccPlatform\n";
		cout << "   OR:\t" << argv[0] << " IH IW ID TH TW TD flccType flccPlatform\n";
		cout << "flccType:\t0 for FLCC_SINGLE, 1 for FLCC_STREAM\n";
		cout << "flccPlatform:\t0 for FLCC_DEVICE, 1 for FLCC_HOST, 2 for FLCC_ANY\n";
		cout << "\n";
		return 1;
	}

	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Variable declarations
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	int dim;
	flccSize imSize, temSize;
	flccType type;
	flccPlatform platform;
	flccResult res;
	flccPlan plan;
	int t;
	
	switch (argc)
	{
	case 7:
		dim = 2;
	
		imSize.h = atoi(argv[1]);
		imSize.w = atoi(argv[2]);
	
		temSize.h = atoi(argv[3]);
		temSize.w = atoi(argv[4]);
		
		t = atoi(argv[5]);
		if      (t==0)  type = FLCC_SINGLE;
		else if (t==1)	type = FLCC_STREAM;
		
		t = atoi(argv[6]);
		if 	    (t==0)	platform = FLCC_DEVICE;
		else if (t==1)	platform = FLCC_HOST;
		else if (t==2)	platform = FLCC_ANY;
		
		break;
		
	case 9:
		dim = 3;
	
		imSize.h = atoi(argv[1]);
		imSize.w = atoi(argv[2]);
		imSize.d = atoi(argv[3]);
	
		temSize.h = atoi(argv[4]);
		temSize.w = atoi(argv[5]);
		temSize.d = atoi(argv[6]);
		
		t = atoi(argv[7]);
		if      (t==0)	type = FLCC_SINGLE;
		else if (t==1)	type = FLCC_STREAM;
		
		t = atoi(argv[8]);
		if 	    (t==0)	platform = FLCC_DEVICE;
		else if (t==1)	platform = FLCC_HOST;
		else if (t==2)	platform = FLCC_ANY;
		
		break;
	}

	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Select convolution method
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	res = conv_plan_d (&plan, dim, imSize, temSize, type, platform);
	
	if (res == FLCC_SUCCESS)
	{
		switch (plan.method)
		{
		case FLCC_DIRECT2_DEV:	
			cout << "FLCC_DIRECT2_DEV\t";
			break;
			
		case FLCC_DIRECT3_DEV:
			cout << "FLCC_DIRECT3_DEV\t";
			break;
			
		case FLCC_FAST2_DEV:
			cout << "FLCC_FAST2_DEV\t";
			break;
			
		case FLCC_FAST3_DEV:
			cout << "FLCC_FAST3_DEV\t";
			break;
			
		case FLCC_DIRECT2_HOST:
			cout << "FLCC_DIRECT2_HOST\t";
			break;
			
		case FLCC_DIRECT3_HOST:
			cout << "FLCC_DIRECT3_HOST\t";
			break;
			
		case FLCC_FAST2_HOST:
			cout << "FLCC_FAST2_HOST\t";
			break;
			
		case FLCC_FAST3_HOST:
			cout << "FLCC_FAST3_HOST\t";
			break;
			
		default:
			cout << "error\t";
		}	
	}
	else
	{
		cout << "error\t";
		exit(1);
	}
	
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Destroy plan
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	res = conv_destroy_d (&plan);
	
	if (res != FLCC_SUCCESS)
	{
		cout << "destroy_error\t";
		exit(1);
	}
	
	
	return 0;
}
